==> AxolotlClient-3.1.6%2B1.20.1-sources.jar: io/github/axolotlclient/api/NewsScreen.java <==
/*
 * Copyright © 2024 moehreag <moehreag@gmail.com> & Contributors
 *
 * This file is part of AxolotlClient.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * For more information, see the LICENSE file.
 */

package io.github.axolotlclient.api;

import java.util.List;
import net.minecraft.class_1144;
import net.minecraft.class_2561;
import net.minecraft.class_332;
import net.minecraft.class_339;
import net.minecraft.class_3532;
import net.minecraft.class_4185;
import net.minecraft.class_437;
import net.minecraft.class_5244;
import net.minecraft.class_5481;
import net.minecraft.class_6381;
import net.minecraft.class_6382;
import io.github.axolotlclient.api.requests.GlobalDataRequest;

public class NewsScreen extends class_437 {


	private final class_437 parent;

	public NewsScreen(class_437 parent) {
		super(class_2561.method_43471("api.notes.title"));

		this.parent = parent;
	}

	@Override
	public void method_25394(class_332 graphics, int mouseX, int mouseY, float delta) {
		method_25420(graphics);
		super.method_25394(graphics, mouseX, mouseY, delta);

		graphics.method_27534(field_22793, field_22785, field_22789 / 2, 20, -1);
	}

	@Override
	protected void method_25426() {
		GlobalDataRequest.get().thenAccept(data -> method_37063(new NewsWidget(25, 35, field_22789 - 50, field_22790 - 100, class_2561.method_43471(data.notes().trim().replaceAll("([^\n])\n([^\n])", "$1 $2")))));

		method_37063(class_4185.method_46430(class_5244.field_24339, buttonWidget -> field_22787.method_1507(parent))
			.method_46434(field_22789 / 2 - 100, field_22790 - 45, 200, 20)
			.method_46431());
	}

	private class NewsWidget extends AbstractTextAreaWidget {

		private final List<class_5481> lines;
		private final int contentHeight;

		public NewsWidget(int x, int y, int width, int height, class_2561 component) {
			super(x, y, width, height, component);
			lines = field_22793.method_1728(method_25369(), method_25368() - 4);
			contentHeight = lines.size() * field_22793.field_2000;
		}

		@Override
		protected int getInnerHeight() {
			return contentHeight;
		}

		@Override
		protected void renderContents(class_332 graphics, int mouseX, int mouseY, float partialTick) {
			int y = method_46427() + 2;
			for (class_5481 chsq : lines) {
				graphics.method_35720(field_22793, chsq, method_46426() + 2, y, -1);
				y += field_22793.field_2000;
			}
		}

		@Override
		protected double scrollRate() {
			return field_22793.field_2000;
		}

		@Override
		protected void method_47399(class_6382 narrationElementOutput) {
			narrationElementOutput.method_37034(class_6381.field_33788, method_25369());
		}

		@Override
		protected void renderBackground(class_332 guiGraphics) {
		}
	}

	public abstract static class AbstractTextAreaWidget extends AbstractScrollArea {
		private static final int INNER_PADDING = 4;

		public AbstractTextAreaWidget(int x, int y, int width, int height, class_2561 component) {
			super(x, y, width, height, component);
		}

		@Override
		public boolean method_25402(double mouseX, double mouseY, int button) {
			boolean bl = this.updateScrolling(mouseX, mouseY, button);
			return super.method_25402(mouseX, mouseY, button) || bl;
		}

		@Override
		public boolean method_25404(int keyCode, int scanCode, int modifiers) {
			boolean bl = keyCode == 265;
			boolean bl2 = keyCode == 264;
			if (bl || bl2) {
				double d = this.scrollAmount();
				this.setScrollAmount(this.scrollAmount() + (double) (bl ? -1 : 1) * this.scrollRate());
				if (d != this.scrollAmount()) {
					return true;
				}
			}

			return super.method_25404(keyCode, scanCode, modifiers);
		}

		@Override
		public void method_48579(class_332 guiGraphics, int mouseX, int mouseY, float partialTick) {
			if (this.field_22764) {
				this.renderBackground(guiGraphics);
				guiGraphics.method_44379(this.method_46426() + 1, this.method_46427() + 1, this.method_46426() + this.field_22758 - 1, this.method_46427() + this.field_22759 - 1);
				guiGraphics.method_51448().method_22903();
				guiGraphics.method_51448().method_22904(0.0, -this.scrollAmount(), 0.0);
				this.renderContents(guiGraphics, mouseX, mouseY, partialTick);
				guiGraphics.method_51448().method_22909();
				guiGraphics.method_44380();
				this.renderDecorations(guiGraphics);
			}
		}

		protected void renderDecorations(class_332 guiGraphics) {
			this.renderScrollbar(guiGraphics);
		}

		protected int innerPadding() {
			return INNER_PADDING;
		}

		protected int totalInnerPadding() {
			return this.innerPadding() * 2;
		}

		@Override
		public boolean method_25405(double mouseX, double mouseY) {
			return this.field_22763
				&& this.field_22764
				&& mouseX >= (double) this.method_46426()
				&& mouseY >= (double) this.method_46427()
				&& mouseX < (double) (this.getXEnd() + 6)
				&& mouseY < (double) this.getYEnd();
		}

		@Override
		protected int scrollBarX() {
			return this.getXEnd();
		}

		@Override
		protected int contentHeight() {
			return this.getInnerHeight() + this.totalInnerPadding();
		}

		protected void renderBackground(class_332 guiGraphics) {
			this.renderBorder(guiGraphics, this.method_46426(), this.method_46427(), this.method_25368(), this.method_25364());
		}

		protected void renderBorder(class_332 guiGraphics, int x, int y, int width, int height) {
			int i = this.method_25370() ? -1 : -6250336;
			guiGraphics.method_25294(this.method_46426(), this.method_46427(), this.getXEnd(), this.getYEnd(), i);
			guiGraphics.method_25294(this.method_46426() + 1, this.method_46427() + 1, this.getXEnd() - 1, this.getYEnd() - 1, -16777216);
		}

		protected boolean withinContentAreaTopBottom(int top, int bottom) {
			return (double) bottom - this.scrollAmount() >= (double) this.method_46427() && (double) top - this.scrollAmount() <= (double) (this.method_46427() + this.field_22759);
		}

		protected abstract int getInnerHeight();

		protected abstract void renderContents(class_332 guiGraphics, int mouseX, int mouseY, float partialTick);

		protected int getInnerLeft() {
			return this.method_46426() + this.innerPadding();
		}

		protected int getInnerTop() {
			return this.method_46427() + this.innerPadding();
		}

		@Override
		public void method_25354(class_1144 handler) {
		}
	}

	public abstract static class AbstractScrollArea extends class_339 {
		public static final int SCROLLBAR_WIDTH = 6;
		private double scrollAmount;
		private boolean scrolling;

		public AbstractScrollArea(int x, int y, int width, int height, class_2561 component) {
			super(x, y, width, height, component);
		}

		@Override
		public boolean method_25401(double mouseX, double mouseY, double scrollY) {
			if (!this.field_22764) {
				return false;
			} else {
				this.setScrollAmount(this.scrollAmount() - scrollY * this.scrollRate());
				return true;
			}
		}

		@Override
		public boolean method_25403(double mouseX, double mouseY, int button, double dragX, double dragY) {
			if (this.scrolling) {
				if (mouseY < (double) this.method_46427()) {
					this.setScrollAmount(0.0);
				} else if (mouseY > (double) this.getYEnd()) {
					this.setScrollAmount(this.maxScrollAmount());
				} else {
					double d = Math.max(1, this.maxScrollAmount());
					int i = this.scrollerHeight();
					double e = Math.max(1.0, d / (double) (this.field_22759 - i));
					this.setScrollAmount(this.scrollAmount() + dragY * e);
				}

				return true;
			} else {
				return super.method_25403(mouseX, mouseY, button, dragX, dragY);
			}
		}

		@Override
		public void method_25357(double mouseX, double mouseY) {
			this.scrolling = false;
		}

		public double scrollAmount() {
			return this.scrollAmount;
		}

		public void setScrollAmount(double scrollAmount) {
			this.scrollAmount = class_3532.method_15350(scrollAmount, 0.0, this.maxScrollAmount());
		}

		public boolean updateScrolling(double mouseX, double mouseY, int button) {
			this.scrolling = this.scrollbarVisible()
				&& this.method_25351(button)
				&& mouseX >= (double) this.scrollBarX()
				&& mouseX <= (double) (this.scrollBarX() + 6)
				&& mouseY >= (double) this.method_46427()
				&& mouseY < (double) this.getYEnd();
			return this.scrolling;
		}

		public void refreshScrollAmount() {
			this.setScrollAmount(this.scrollAmount);
		}

		public int maxScrollAmount() {
			return Math.max(0, this.contentHeight() - this.field_22759);
		}

		protected boolean scrollbarVisible() {
			return this.maxScrollAmount() > 0;
		}

		protected int scrollerHeight() {
			return class_3532.method_15340((int) ((float) (this.field_22759 * this.field_22759) / (float) this.contentHeight()), 32, this.field_22759 - 8);
		}

		protected int scrollBarX() {
			return this.getXEnd() - 6;
		}

		protected int scrollBarY() {
			return Math.max(this.method_46427(), (int) this.scrollAmount * (this.field_22759 - this.scrollerHeight()) / this.maxScrollAmount() + this.method_46427());
		}

		protected void renderScrollbar(class_332 guiGraphics) {
			if (this.scrollbarVisible()) {
				int i = this.scrollBarX();
				int j = this.scrollerHeight();
				int k = this.scrollBarY();
				guiGraphics.method_25294(i, method_46427(), i + SCROLLBAR_WIDTH, getYEnd(), -16777216);
				guiGraphics.method_25294(i, k, i + SCROLLBAR_WIDTH, k + j, -8355712);
				guiGraphics.method_25294(i, k, i + SCROLLBAR_WIDTH - 1, k + j - 1, -4144960);
			}
		}

		protected int getYEnd() {
			return method_46427() + method_25364();
		}

		protected abstract int contentHeight();

		protected abstract double scrollRate();

		protected int getXEnd() {
			return method_46426() + method_25368();
		}
	}
}

==> AxolotlClient-3.1.7%2B1.21.11-sources.jar: io/github/axolotlclient/api/NewsScreen.java <==
/*
 * Copyright © 2024 moehreag <moehreag@gmail.com> & Contributors
 *
 * This file is part of AxolotlClient.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * For more information, see the LICENSE file.
 */

package io.github.axolotlclient.api;

import java.util.List;
import net.minecraft.class_10415;
import net.minecraft.class_2561;
import net.minecraft.class_332;
import net.minecraft.class_4185;
import net.minecraft.class_437;
import net.minecraft.class_5244;
import net.minecraft.class_5481;
import net.minecraft.class_6381;
import net.minecraft.class_6382;
import io.github.axolotlclient.api.requests.GlobalDataRequest;

public class NewsScreen extends class_437 {

	private final class_437 parent;

	public NewsScreen(class_437 parent) {
		super(class_2561.method_43471("api.notes.title"));
		this.parent = parent;
	}

	@Override
	public void method_25394(class_332 graphics, int mouseX, int mouseY, float delta) {
		super.method_25394(graphics, mouseX, mouseY, delta);

		graphics.method_27534(field_22793, field_22785, field_22789 / 2, 20, -1);
	}

	@Override
	protected void method_25426() {
		GlobalDataRequest.get().thenAccept(data -> method_37063(new NewsWidget(25, 35, field_22789 - 50, field_22790 - 100, class_2561.method_43470(data.notes().trim().replaceAll("([^\n])\n([^\n])", "$1 $2")))));
		method_37063(class_4185.method_46430(class_5244.field_24339, buttonWidget -> field_22787.method_1507(parent)).method_46434(field_22789 / 2 - 100, field_22790 - 45, 200, 20).method_46431());
	}

	private class NewsWidget extends class_10415 {

		private final List<class_5481> lines;
		private final int contentHeight;

		public NewsWidget(int x, int y, int width, int height, class_2561 component) {
			super(x, y, width, height, component);
			lines = field_22793.method_1728(method_25369(), method_25368() - 4);
			contentHeight = lines.size() * field_22793.field_2000;
		}

		@Override
		protected int method_44391() {
			return contentHeight;
		}

		@Override
		protected void method_44389(class_332 graphics, int mouseX, int mouseY, float partialTick) {
			int y = method_46427() + 2;
			for (class_5481 chsq : lines) {
				graphics.method_35720(field_22793, chsq, method_46426() + 2, y, -1);
				y += field_22793.field_2000;
			}
		}

		@Override
		protected double method_44393() {
			return field_22793.field_2000;
		}

		@Override
		protected void method_47399(class_6382 narrationElementOutput) {
			narrationElementOutput.method_37034(class_6381.field_33788, method_25369());
		}

		@Override
		protected void method_44386(class_332 guiGraphics) {
		}
	}
}
